/* ROSA Media Player Plugin
 * Julia Mineeva, Evgeniy Augin. Copyright (c) 2011 ROSA  <support@rosalab.ru>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 3,
 *   or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details
 *
 *   You should have received a copy of the GNU General Public
 *   License along with this program; if not, write to the
 *   Free Software Foundation, Inc.,
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include "controlpanelform.h"
#include "ui_controlpanelform.h"

#include <QPalette>
#include <QAction>
#include <QRegion>
#include <QMenu>
#include <QLabel>
#include <QMovie>
#include <QDebug>
#include <QLinearGradient>
#include <QPainter>
#include <QPixmap>
#include <QRect>
#include <QColor>
#include <QTime>
#include <QtSvg/QSvgRenderer>
#include <QPixmap>
#include <QImage>

#include <math.h>

#include "global.h"
#include "preferences.h"
#include "waitwidget.h"

using namespace Global;

ControlPanelForm::ControlPanelForm( Core* core, QWidget* parent )
    : QWidget( parent )
    , ui( new Ui::ControlPanelForm )
    , m_core( core )
    , m_menu( 0 )
    , m_waitWidget( 0 )
    , m_isSeeking( false )
{
    ui->setupUi( this );

    setAutoFillBackground( true );

    QPalette pal = palette();
    pal.setColor( QPalette::Window, Qt::black );
    setPalette( pal );

    // Init play button control
    ui->playButton->setIcon( QIcon( renderSvg( QString( ":play" ), /*ui->playButton->rect()*/ QRect( 0, 0, 25, 25 ) ) ) );
    ui->playButton->setEnabled( false );

    connect( ui->playButton, SIGNAL( clicked( bool ) ), this, SLOT( playClicked() ) );
    connect( m_core, SIGNAL( stateChanged( Core::State ) ), this,  SLOT( updatePlayButton( Core::State ) ) );
    connect( m_core, SIGNAL( mediaLoaded() ), this, SLOT( enableActionsOnPlaying() ) );
    connect( m_core, SIGNAL( mediaFinished() ), this , SLOT( disableActionsOnStop() ), Qt::QueuedConnection );
    connect( m_core, SIGNAL( mediaStoppedByUser()), this, SLOT( disableActionsOnStop() ) );

    // Init volume control
    ui->volumeDownButton->setIcon( QIcon( ":minpng" ) );
    ui->volumeUpButton->setIcon( QIcon( ":maxpng" ) );
    ui->volumeDownButton->setEnabled( false );
    ui->volumeUpButton->setEnabled( false );

    ui->volumeSlider->setMinimum( 0 );
    ui->volumeSlider->setMaximum( 100 );
    ui->volumeSlider->setOrientation( Qt::Horizontal );
    ui->volumeSlider->setSizePolicy( QSizePolicy::Fixed, QSizePolicy::Fixed );
    ui->volumeSlider->setFocusPolicy( Qt::NoFocus );
    ui->volumeSlider->setTickPosition( QSlider::NoTicks );
    ui->volumeSlider->setTickInterval( 10 );
    ui->volumeSlider->setSingleStep( 1 );
    ui->volumeSlider->setPageStep( 10 );
    ui->volumeSlider->setToolTip( tr( "Volume" ) );
    ui->volumeSlider->setAttribute( Qt::WA_NoMousePropagation );
    ui->volumeSlider->setEnabled( false );
    ui->volumeSlider->setValue( m_core->mset.volume );

    connect( ui->volumeSlider, SIGNAL( valueChanged( int ) ), this, SLOT( updateVolume( int ) ) );
    connect( ui->volumeDownButton, SIGNAL( clicked( bool ) ), this, SLOT( volumeMute() ) );
    connect( ui->volumeUpButton, SIGNAL( clicked( bool ) ), this, SLOT( volumeFull() ) );
    connect( m_core, SIGNAL( volumeChanged( int ) ), this, SLOT( updateVolumeSlider( int ) ) );

    // Init time slider
#ifdef SEEKBAR_RESOLUTION
    connect( ui->timeSlider, SIGNAL( posChanged( int ) ), this, SLOT( goToPosition( int ) ) );
    connect( m_core, SIGNAL( positionChanged( int ) ), ui->timeSlider, SLOT( setPos( int ) ) );
#else
    connect( ui->timeSlider, SIGNAL( posChanged( int ) ), m_core, SLOT( goToPos( int ) ) );
    connect( m_core, SIGNAL(posChanged( int ) ), ui->timeSlider, SLOT( setPos( int ) ) );
#endif

#if ENABLE_DELAYED_DRAGGING
    ui->timeSlider->setDragDelay( pref->time_slider_drag_delay );
    connect( ui->timeSlider, SIGNAL( delayedDraggingPos( int ) ),  this, SLOT( goToPosOnDragging( int ) ) );
#else
    connect( ui->timeSlider, SIGNAL( draggingPos( int ) ), this, SLOT( goToPosOnDragging( int ) ) );
#endif
    connect( m_core, SIGNAL( showTime( double ) ), this, SLOT( updateTimeSlider( double ) ) );

    // Menu for size widget control
    m_menu = new QMenu( this );
    m_menu->setStyleSheet( "QMenu {color: white; background-color: qlineargradient(spread:pad, x1:0, y1:1, x2:0, y2:0, stop:0 rgba(20, 20, 20, 255), stop:1 rgba(60, 60, 60, 255)); margin: 2px;}"
                           "QMenu::item {padding: 2px 25px 2px 20px; border: 1px solid transparent;}"
                           "QMenu::item:selected { border-color: gray; background: rgba(100, 100, 100, 150);}");

    // create actions and connections for plugin size menu
    QAction* setLargeSize = m_menu->addAction( QIcon( ":/images/sizelarge-active.png" ), tr( "Large" ) );
    connect( setLargeSize, SIGNAL ( triggered( bool ) ), this, SLOT(setLargeSize() ) );
    QAction* setMediumSize = m_menu->addAction( QIcon( ":/images/sizemedium-active.png" ), tr( "Medium" ) );
    connect( setMediumSize, SIGNAL ( triggered(bool)), this, SLOT(setMediumSize() ) );
    QAction* setSmallSize = m_menu->addAction( QIcon( ":/images/sizesmall-active.png" ), tr( "Small" ) );
    connect( setSmallSize, SIGNAL ( triggered( bool ) ), this, SLOT( setSmallSize() ) );

    // Init size and fullscreen buttons
    ui->changeSizeButton->setIcon( QIcon( ":changepng" ) );
    ui->fullScreenButton->setIcon( QIcon( ":fullpng" ) );
    ui->fullScreenButton->setAutoFillBackground(true);

    connect( ui->changeSizeButton, SIGNAL( clicked() ), this, SLOT( showSizeMenu() ) );
    connect( ui->fullScreenButton, SIGNAL( clicked() ), this, SLOT( toggleFullScreen() ) );

    // process message with cache info
    connect(m_core, SIGNAL( showMessage( QString ) ), this, SLOT( cacheMessage( QString ) ) );

    // create wait widget
    m_waitWidget = new WaitWidget( parentWidget() );
    Q_ASSERT( m_waitWidget != 0 );
    m_waitWidget->hide();
}

ControlPanelForm::~ControlPanelForm()
{
    delete ui;

    if ( m_menu )
        delete m_menu;

    if ( m_waitWidget )
        delete m_waitWidget;
}

void ControlPanelForm::setParentWidget( QWidget* parent )
{
    this->setParent( parent );
    if ( m_waitWidget )
        m_waitWidget->setParent( parent );
}

/* set new position of time slider handle */
void ControlPanelForm::goToPosition( int pos )
{
    ui->timeSlider->setPos( pos );

    // make checking for stream type because stream is slow
    // and we must show animation while download video
    if ( m_core->mdat.type != TYPE_STREAM )
    {
        // simply make seek
        m_core->goToPosition( pos );
        return;
    }

    m_isSeeking = true;

    // wait "PLAY" event
    disconnect( m_core, SIGNAL( stateChanged( Core::State ) ), this,  SLOT( updatePlayButton( Core::State ) ) );
    connect( m_core, SIGNAL( stateChanged( Core::State ) ), this,  SLOT( receiveStateChangedEvent( Core::State ) ) );

    // set video to pause
    m_core->pause();

    // set to new seek position
    m_core->goToPosition( pos );

    // block time slider until come "PLAY" event
    ui->timeSlider->setEnabled( false );
    ui->changeSizeButton->setEnabled( false );

    // show animation widget
    m_waitWidget->move( ( parentWidget()->width() - m_waitWidget->width() ) / 2, ( parentWidget()->height() - m_waitWidget->height() ) / 2 );
    m_waitWidget->show();
    m_waitWidget->raise();
}

/* restore state of time slider */
void ControlPanelForm::receiveStateChangedEvent( Core::State state )
{
    if ( state != Core::Playing && !m_isSeeking )
        return;

    // working as usually
    disconnect( m_core, SIGNAL( stateChanged( Core::State ) ), this,  SLOT( receiveStateChangedEvent( Core::State ) ) );
    connect( m_core, SIGNAL( stateChanged( Core::State ) ), this,  SLOT( updatePlayButton( Core::State ) ) );

    // unblock time slider
    ui->timeSlider->setEnabled( true );
    ui->changeSizeButton->setEnabled( true );

    // hide animation widget
    m_waitWidget->hide();

    m_isSeeking = false;
}

void ControlPanelForm::paintEvent( QPaintEvent* e )
{
    Q_UNUSED( e );

    QPixmap p = renderSvg(QString(":/background"),rect() ) ;
    QImage image = p.toImage();

    QPainter painter( this );
    painter.drawImage( rect(), image );
}

void ControlPanelForm::updatePlayButton( Core::State state )
{
    if ( state == Core::Playing )
        ui->playButton->setIcon( QIcon( renderSvg( QString( ":pause" ), QRect(QPoint(), ui->playButton->iconSize() ) ) ) );
    else
        ui->playButton->setIcon( QIcon( renderSvg( QString( ":play" ), QRect(QPoint(), ui->playButton->iconSize() ) ) ) );

    repaint();
}

void ControlPanelForm::goToPosOnDragging( int t )
{
    if ( pref->update_while_seeking )
    {
#if ENABLE_DELAYED_DRAGGING
#ifdef SEEKBAR_RESOLUTION
        m_core->goToPosition( t );
#else
        m_core->goToPos( t );
#endif
#else
        if ( ( t % 4 ) == 0 )
        {
            qDebug("BaseGui::goToPosOnDragging: %d", t);
#ifdef SEEKBAR_RESOLUTION
            m_core->goToPosition( t );
#else
        m_core->goToPos( t );
#endif
        }
#endif
    }
}

/* update time information on the control panel */
void ControlPanelForm::updateTimeSlider( double sec )
{
    static int last_second = 0;

    if ( floor (sec) == last_second ) return;
    last_second = ( int ) floor( sec );

    // processed time
    QTime t( 0, 0, 0 );
    t = t.addSecs( ( int ) sec );
    QString str = ( sec >= 3600 ) ? "hh:mm:ss" : "mm:ss";
    ui->timeLabel->setText( t.toString( str ) );

    // rest time
    t.setHMS( 0, 0, 0 );
    t = t.addSecs( ( int ) ( m_core->mdat.duration - sec ) );
    if ( ( m_core->mdat.duration - sec ) >= 3600 )
    {
        str = ( m_core->mdat.type == TYPE_STREAM ) ? "-00:00" : "-hh:mm:ss";
        ui->lastTimeLabel->setText( str );
    }
    else
        ui->lastTimeLabel->setText( t.toString( "-mm:ss" ) );
}

void ControlPanelForm::volumeMute()
{
    m_core->setVolume( 0 );
}

void ControlPanelForm::volumeFull()
{
    m_core->setVolume( 100 );
}

void ControlPanelForm::updateVolumeSlider( int v )
{
    ui->volumeSlider->setValue( v );
    ui->volumeUpButton->setEnabled( v != 100 );
    ui->volumeDownButton->setEnabled( v != 0 );
}

void ControlPanelForm::updateVolume( int v )
{
    m_core->setVolume( v );
}

/* set boreder radius for control panel */
void ControlPanelForm::setBorderRadius(int radius)
{
    radius = 7;
    QRegion region( 0, 0, width(), height(), QRegion::Rectangle );

    // top left
    QRegion round ( 0, 0, 2*radius, 2*radius, QRegion::Ellipse );
    QRegion corner( 0, 0, radius, radius, QRegion::Rectangle );
    region = region.subtracted( corner.subtracted( round ) );

    // top right
    round  = QRegion( width() - 2 * radius, 0, 2 * radius, 2 * radius, QRegion::Ellipse );
    corner = QRegion( width() - radius, 0, radius, radius, QRegion::Rectangle );
    region = region.subtracted( corner.subtracted( round ) );

    // bottom right
    round  = QRegion( width() - 2 * radius, height() - 2 * radius, 2 * radius, 2 * radius, QRegion::Ellipse );
    corner = QRegion( width() - radius, height() - radius, radius, radius, QRegion::Rectangle );
    region = region.subtracted( corner.subtracted( round ) );

    // bottom left
    round  = QRegion( 0, height() - 2 * radius, 2 * radius, 2 * radius, QRegion::Ellipse );
    corner = QRegion( 0, height() - radius, radius, radius, QRegion::Rectangle );
    region = region.subtracted( corner.subtracted( round ) );

    //setMask( region );
}

/* change play state */
void ControlPanelForm::playClicked()
{
    if ( !m_core ) return;
    if ( m_core->state() == Core::Playing )
        m_core->pause();
    else
        m_core->play();
}

void ControlPanelForm::enableActionsOnPlaying()
{
    ui->playButton->setEnabled( true );
    ui->timeSlider->setEnabled( true );
    ui->volumeDownButton->setEnabled( true );
    ui->volumeUpButton->setEnabled( true );
    ui->volumeSlider->setEnabled( true );
}

void ControlPanelForm::disableActionsOnStop()
{
    ui->timeSlider->setEnabled( false );
    ui->volumeDownButton->setEnabled( false );
    ui->volumeUpButton->setEnabled( false );
    ui->volumeSlider->setEnabled( false );
}

void ControlPanelForm::setLargeSize()
{
    emit setResolution( 3 );
}

void ControlPanelForm::setMediumSize()
{
    emit setResolution( 2 );
}

void ControlPanelForm::setSmallSize()
{
    emit setResolution( 1 );
}

void ControlPanelForm::showSizeMenu()
{
    QPoint point = ui->changeSizeButton->mapTo( this, ui->changeSizeButton->rect().center() );
    QSize size = m_menu->sizeHint();

    point.setY( rect().y() - size.height() - 3 );
    point.setX( point.x() - size.width() / 2 );

    m_menu->popup( mapToGlobal( point ) );
}

void ControlPanelForm::toggleFullScreen()
{
    emit setFullScreen();
}

void ControlPanelForm::toggleFullScreenMode( bool enable )
{
    ui->changeSizeButton->setVisible( !enable );

    if ( enable )
    {
        ui->horizontalSpacer_3->changeSize( 70 ,20,QSizePolicy::Fixed,QSizePolicy::Fixed );
        ui->fullScreenButton->setIcon( QIcon( ":resumepng" ) );
    }
    else
    {
        ui->horizontalSpacer_3->changeSize(  30,20,QSizePolicy::Fixed,QSizePolicy::Fixed );
        ui->fullScreenButton->setIcon( QIcon( ":fullpng" ) );
    }
}

void ControlPanelForm::cacheMessage( QString str )
{
    if ( str.contains( "Cache fill" ) )
    {
        str  = str.mid( 12, 2 );
        //int res = str.toInt();
        //ui->timeSlider->setCacheValue(res);
    }
}

void ControlPanelForm::resizeEvent ( QResizeEvent * e)
{
    QWidget::resizeEvent( e );

    // update wait widget position
    m_waitWidget->move( ( parentWidget()->width() - m_waitWidget->width() ) / 2, ( parentWidget()->height() - m_waitWidget->height() ) / 2 );
    if ( m_waitWidget->isVisible() )
        m_waitWidget->raise();
}

QPixmap ControlPanelForm::renderSvg( QString svg, QRect rect )
{
    QPixmap pix( rect.width(), rect.height() );
    pix.fill( Qt::transparent );

    QSvgRenderer svgr( svg );

    QPainter p( &pix );
    p.setRenderHint( QPainter::Antialiasing );
    svgr.render( &p, rect );
    p.end();

    return pix;
}
