/* ROSA Media Player Plugin
 * Julia Mineeva, Evgeniy Augin. Copyright (c) 2011 ROSA  <support@rosalab.ru>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 3,
 *   or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details
 *
 *   You should have received a copy of the GNU General Public
 *   License along with this program; if not, write to the
 *   Free Software Foundation, Inc.,
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include "minicontrolpanel.h"
#include "ui_minicontrolpanel.h"
#include "myslider.h"
#include "global.h"
#include "preferences.h"
#include "volumebutton.h"

#include <QtSvg/QSvgRenderer>
#include <QPainter>
#include <QPixmap>
#include <QImage>
#include <QTime>
#include <QBoxLayout>
#include <QDebug>
#include <QTimer>

#include <math.h>

using namespace Global;

MiniControlPanel::MiniControlPanel( Core* core, QWidget *parent )
    : QWidget( parent )
    , ui( new Ui::MiniControlPanel )
    , m_core( core )
{
    ui->setupUi( this );

    setMouseTracking( true );

    // Init Play Button
    ui->playButton->setIcon( QIcon( renderSvg( QString( ":play" ), QRect( QPoint(), ui->playButton->iconSize() ) ) ) );
    ui->playButton->setEnabled( false );

    connect( ui->playButton, SIGNAL( clicked( bool ) ), this, SLOT( playClicked() ) );
    connect( ui->playButton, SIGNAL( clicked() ), this, SLOT( hideVolumeControl() ) );

    connect( m_core, SIGNAL( stateChanged( Core::State ) ), this,  SLOT( updatePlayButton( Core::State ) ) );
    connect( m_core, SIGNAL( mediaLoaded() ), this, SLOT( enableActionsOnPlaying() ) );
    connect( m_core, SIGNAL( mediaFinished() ), this , SLOT( disableActionsOnStop() ), Qt::QueuedConnection );
    connect( m_core, SIGNAL( mediaStoppedByUser()), this, SLOT( disableActionsOnStop() ) );

    // Init Volume Button
    ui->volumeButton->setIcon( QIcon( ":mini.sound.png" ) );

    connect( ui->volumeButton, SIGNAL( clicked() ), this, SLOT( toogleVolumeControl() ) );
    connect( ui->volumeButton, SIGNAL( wheelValueChanged( int ) ), this, SLOT( volumeWheelControl( int ) ) );

    // Init resolution button
    ui->resolutionButton->setIcon( QIcon( ":mini.res.png" ) );
    ui->resolutionButton->setVisible(false);

    // Init fullscreen button
    ui->fullScreenButton->setIcon( QIcon( ":mini.full.png" ) );

    connect( ui->fullScreenButton, SIGNAL( clicked() ), this, SLOT( toggleFullScreen() ) );
    connect( ui->fullScreenButton, SIGNAL( clicked() ), this, SLOT( hideVolumeControl() ) );

    // Init time slider
    ui->timeSlider->setDragDelay( pref->time_slider_drag_delay );

    connect( ui->timeSlider, SIGNAL( clicked()), this, SLOT( hideVolumeControl() ) );
    connect( ui->timeSlider, SIGNAL( posChanged( int ) ), this, SLOT( goToPosition( int ) ) );
    connect( ui->timeSlider, SIGNAL( delayedDraggingPos( int ) ),  this, SLOT( goToPosOnDragging( int ) ) );

    connect( m_core, SIGNAL( positionChanged( int ) ), ui->timeSlider, SLOT( setPos( int ) ) );
    connect( m_core, SIGNAL( showTime( double ) ), this, SLOT( updateTimeSlider( double ) ) );

    // timer for seek command bug (turn on when does'not come reply from mplayer)
    m_hideWaitWidgetTimer = new QTimer( this );
    connect( m_hideWaitWidgetTimer, SIGNAL( timeout() ), this, SLOT( hideWaitWidgetSlot() ) );
}

MiniControlPanel::~MiniControlPanel()
{
    delete ui;

    if ( m_hideWaitWidgetTimer )
        delete m_hideWaitWidgetTimer;
}

void MiniControlPanel::paintEvent(QPaintEvent *)
{
    QPixmap p = renderSvg(QString(":/mini.back"), rect() ) ;
    QImage image = p.toImage();

    QPainter painter( this );
    painter.drawImage( rect(), image );
}

/* set new position of time slider handle */
void MiniControlPanel::toogleVolumeControl()
{
    QPoint point = ui->volumeButton->mapToGlobal( ui->volumeButton->rect().center() );
    emit showVolumeSlider( point.x(), point.y() );
}

void MiniControlPanel::goToPosition( int pos )
{
    ui->timeSlider->setPos( pos );

    // make checking for stream type because stream is slow
    // and we must show animation while download video
    if ( m_core->mdat.type != TYPE_STREAM )
    {
        // simply make seek
        m_core->goToPosition( pos );
        return;
    }

    // wait "PLAY" event
    disconnect( m_core, SIGNAL( stateChanged( Core::State ) ), this,  SLOT( updatePlayButton( Core::State ) ) );

    // set video to pause
    m_core->pause();

    // set to new seek position
    m_core->goToPosition( pos );

    // block time slider until come "PLAY" event
    ui->timeSlider->setEnabled( false );
//    ui->playButton->setEnabled( false );
    ui->resolutionButton->setEnabled( false );

    // show animation widget
    m_isSeeking = true;
    emit isSeeking( m_isSeeking );

    connect( m_core, SIGNAL( stateChanged( Core::State ) ), this,  SLOT( receiveStateChangedEvent( Core::State ) ) );

    if ( m_hideWaitWidgetTimer->isActive() )
        m_hideWaitWidgetTimer->stop();
    m_hideWaitWidgetTimer->start( 2000 );
}

/* restore state of time slider */
void MiniControlPanel::receiveStateChangedEvent( Core::State state )
{
    /*
    if ( state == Core::Paused )
    {
        qDebug("MiniControlPanel::receiveStateChangedEvent:     %s:   m_core->state() is %s",
               QTime::currentTime().toString("hh:mm:ss.zzz").toLocal8Bit().data(),
               m_core->stateToString().toLocal8Bit().data() );
    }
*/
    if ( state != Core::Playing )
        return;
//    if ( !m_isSeeking )
//        return;

    // working as usually
    disconnect( m_core, SIGNAL( stateChanged( Core::State ) ), this,  SLOT( receiveStateChangedEvent( Core::State ) ) );

    updatePlayButton( state );

    // unblock time slider
    ui->timeSlider->setEnabled( true );
//    ui->playButton->setEnabled( true );s
    ui->resolutionButton->setEnabled( true );

    // hide animation widget
    m_isSeeking = false;
    emit isSeeking( m_isSeeking );

    connect( m_core, SIGNAL( stateChanged( Core::State ) ), this,  SLOT( updatePlayButton( Core::State ) ) );

    /*
    qDebug(" MiniControlPanel::receiveStateChangedEvent:    %s:   m_core->state() is %s",
           QTime::currentTime().toString("hh:mm:ss.zzz").toLocal8Bit().data(),
           m_core->stateToString().toLocal8Bit().data() );
*/
    if ( m_hideWaitWidgetTimer->isActive() )
        m_hideWaitWidgetTimer->stop();
}

/* update time information on the control panel */
void MiniControlPanel::updateTimeSlider( double sec )
{
    static int last_second = 0;

    if ( floor (sec) == last_second ) return;
    last_second = ( int ) floor( sec );

    // processed time
    QTime t( 0, 0, 0 );
    t = t.addSecs( ( int ) sec );
    QString str = ( sec >= 3600 ) ? "hh:mm:ss" : "mm:ss";
    ui->timeLabel->setText( t.toString( str ) );

    // rest time
    t.setHMS( 0, 0, 0 );
    t = t.addSecs( ( int ) ( m_core->mdat.duration ) );
    if ( ( m_core->mdat.duration ) >= 3600 )
    {
        str = ( m_core->mdat.type == TYPE_STREAM ) ? "00:00" : "hh:mm:ss";
        ui->allTimeLabel->setText( str );
    }
    else
        ui->allTimeLabel->setText( t.toString( "mm:ss" ) );
}

void MiniControlPanel::goToPosOnDragging( int t )
{
    if ( pref->update_while_seeking )
    {
        m_core->goToPosition( t );
    }
}

/* change play state */
void MiniControlPanel::playClicked()
{
    if ( !m_core ) return;
    if ( m_core->state() == Core::Playing )
        m_core->pause();
    else
        m_core->play();
}

void MiniControlPanel::updatePlayButton( Core::State state )
{
    if ( state == Core::Playing )
        ui->playButton->setIcon( QIcon( renderSvg( QString( ":pause" ), QRect( 0, 0, 16, 16 ) ) ) );
    else
        ui->playButton->setIcon( QIcon( renderSvg( QString( ":play" ), QRect( QPoint(), ui->playButton->iconSize() ) ) ) );

    repaint();
}

QPixmap MiniControlPanel::renderSvg( QString svg, QRect rect )
{
    QPixmap pix( rect.width(), rect.height() );
    pix.fill( Qt::transparent );

    QSvgRenderer svgr( svg );

    QPainter p( &pix );
    p.setRenderHint( QPainter::Antialiasing );
    svgr.render( &p, rect );
    p.end();

    return pix;
}

void MiniControlPanel::disableActionsOnStop()
{
    ui->timeSlider->setEnabled( false );
}

void MiniControlPanel::enableActionsOnPlaying()
{
    ui->playButton->setEnabled( true );
    ui->timeSlider->setEnabled( true );
}

void MiniControlPanel::toggleFullScreen()
{
    emit setFullScreen();
}

void MiniControlPanel::setVolumeIcon( int value )
{
    switch ( value )
    {
    case 0:
        ui->volumeButton->setIcon( QIcon( ":sound0.png" ) );
        break;
    case 1:
        ui->volumeButton->setIcon( QIcon( ":sound1.png" ) );
        break;
    case 2:
        ui->volumeButton->setIcon( QIcon( ":sound2.png" ) );
        break;
    case 3:
        ui->volumeButton->setIcon( QIcon( ":mini.sound.png" ) );
        break;
    }
}

void MiniControlPanel::toggleFullScreenMode(bool enable)
{
    if ( enable )
    {
        ui->fullScreenButton->setIcon( QIcon( ":mini.resume.png" ) );
    }
    else
    {
        ui->fullScreenButton->setIcon( QIcon( ":mini.full.png" ) );
    }
}

void MiniControlPanel::volumeWheelControl(int value)
{
    int currentValue = ( pref->global_volume ? pref->volume : m_core->mset.volume );
    m_core->setVolume( currentValue + value );
}

void MiniControlPanel::hideVolumeControl()
{
    emit hideVolumeWindow();
}


void MiniControlPanel::setSliderEnabled( bool enable )
{
    ui->timeSlider->setEnabled( enable );
}


void MiniControlPanel::hideWaitWidgetSlot()
{
    if ( m_isSeeking && ( m_core->state() == Core::Playing ) )
    {
        m_hideWaitWidgetTimer->stop();
        receiveStateChangedEvent( m_core->state() );
    }
    else
    {
        m_hideWaitWidgetTimer->start( 2000 );
    }
}
